import java.util.ArrayList;
import java.util.PriorityQueue;

/**
 * Die Klasse Kruskal fuehrt das Generieren eines 
 * minimal spanning tree nach dem
 * Algorithmus von Kruskal durch
 * 
 * @author claus albowski
 * @version Juni 2015
 */
public class Kruskal
{
    private ArrayList<Teilbaum> teilbaeume;
    private PriorityQueue<Kante> sortierte;
    private Graph graph;
    private int anzahl;

    /**
     * Konstruktor für Objekte der Klasse Kruskal
     */
    public Kruskal(int anzahl, int breite, int hoehe)
    {
        this.anzahl = anzahl;
        graph = new Graph(anzahl, breite, hoehe);
        graph.vollstaendig();
        graph.zeichne();
        initialisiereTeilbaeume();
        sortierte = new PriorityQueue<Kante>(Kante.gibKanten());
    }


    /**
     * initialisiereTeilbaeume
     */
    private void initialisiereTeilbaeume() {
        teilbaeume = new ArrayList();
        for (Knoten knoten : Knoten.gibAlleKnoten()) 
            teilbaeume.add(new Teilbaum(knoten));
    }
    
    /**
     * verknuepft alle Knoten zu einem Graphen.
     */
    public void vollstaendig(){
        Kante.loescheKanten();
        for (int i=0;i<anzahl;i++)
        for (int j=i+1;j<anzahl;j++)
        new Kante(Knoten.gibKnoten(i),Knoten.gibKnoten(j));
    }
    
    /**
     * Loesung fuer Algorithmus von Kruskal
     */
    public void minimal()
    {
        sortierte = new PriorityQueue<Kante>(Kante.gibKanten());
        Kante neu;
        Teilbaum erster,zweiter,neuer;
        while (sortierte.size()>0) {
            neu = sortierte.poll();
            erster=null;
            zweiter=null;
            for (Teilbaum teilbaum : teilbaeume) {
                if (teilbaum.hatKnoten(neu.gibVon())) erster=teilbaum;
                if (teilbaum.hatKnoten(neu.gibNach())) zweiter=teilbaum;
            }
            //System.out.println(sortierte.size()+" "+teilbaeume.size());
            if (erster==null) continue;
            if (zweiter==null) continue;
            if (erster==zweiter) continue;
            neuer = new Teilbaum(erster,zweiter,neu);
            zeichne(neu, "rot");
            teilbaeume.add(neuer);
            teilbaeume.remove(erster);
            teilbaeume.remove(zweiter);
        }
    }
    
    /**
     * ausgabe
     */
    public void ausgabe() {
        ArrayList<Kante> alleKanten = teilbaeume.get(0).gibKanten();
        for (Kante kante : alleKanten)
        System.out.println("("+kante.gibVon().gibKnotenNummer()+" "+
                           kante.gibNach().gibKnotenNummer()+")");
    }


    /**
     * Zeichnet diese Kanten des minimalen Graphen mit farbe ein
     */
    public void zeichne(Kante kante, String farbe)
    {
        Leinwand leinwand = Leinwand.gibLeinwand();
        leinwand.zeichne (
            kante,           // leinwand kennt das Objekt
            farbe,           // definiert seine Zeichenfarbe
            kante);          // definiert seinen grafischen Aspekt
    }

}
